/*********************************************************************
 *
 *             Microchip USB C18 Firmware -  WinUsb Demo
 *
 *********************************************************************
 * FileName:        winusb.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original HID mouse firmware (hid.c).
 * Jan Axelson           2/20/08    Modified and renamed for WinUSB device.	
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "user\user_winusb.h"                        
#ifdef USB_USE_WINUSB

/** V A R I A B L E S ********************************************************/
#pragma udata     
byte winusb_bulk_rx_len;
byte winusb_interrupt_rx_len;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void WinusbControlReadTransferHandler(void);
void WinusbControlWriteTransferHandler(void);
void WinusbCtrlTrfTxService(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code

/** C L A S S  S P E C I F I C  R E Q ****************************************/
/******************************************************************************
 * Function:        void USBCheckWinUsbRequest(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine checks the setup data packet to see if it
 *                  knows how to handle it
 *
 * Note:            None
 *****************************************************************************/
void USBCheckWinUsbRequest(void)
{

	// The packet must be directed to an interface with the same ID as the 
	// device's WinUSB interface.

    if(SetupPkt.Recipient != RCPT_INTF) return;
    if(SetupPkt.bIntfID != WINUSB_INTF_ID) return;    
  
    if(SetupPkt.RequestType != VENDOR) return;

	// WinUSB-specific requests.

    switch(SetupPkt.bRequest)
    {
        case WINUSB_REQUEST_1:		
			
            WinusbControlWriteTransferHandler();
            break;

        case WINUSB_REQUEST_2:
			
            WinusbControlReadTransferHandler();            
            break;
      
    }//end switch(SetupPkt.bRequest)

}//end USBCheckWinUsbRequest


/******************************************************************************
 * Function:        byte WinUsbBulkRx(char *buffer)
 *
 * PreCondition:    Input argument 'buffer' should point to a buffer area that
 *                  is bigger or equal to the size specified by 'len'.
 *
 * Input:           buffer  : Pointer to where received bytes are to be stored
 *
 * Output:          The number of bytes copied to buffer.
 *
 * Side Effects:    Publicly accessible variable winusb_bulk_rx_len is updated
 *                  with the number of bytes copied to buffer.
 *                  Once WinUsbBulkRx is called, subsequent retrieval of
 *                  winusb_bulk_rx_len can be done by calling macro
 *                  mWinUsbGetBulkRxLength().
 *
 * Overview:        WinUsbBulkRx copies a string of bytes received on a 
 *                  bulk OUT endpoint to a user's specified location. 
 *                  It is a non-blocking function. It does not wait
 *                  for data if there is no data available. Instead it returns
 *                  '0' to notify the caller that there is no data available.
 *
 * Note:            
 *****************************************************************************/

byte WinUsbBulkRx(char *buffer)
{
	byte	len;
    winusb_bulk_rx_len = 0;
    
    if(!mWinUsbBulkRxIsBusy())
    {
       // Get the number of received bytes.

		len = WINUSB_BD_BULK_OUT.Cnt;      
        
        /*
         * Copy data from dual-ram buffer to user's buffer
         */

        for(winusb_bulk_rx_len = 0; winusb_bulk_rx_len < len; winusb_bulk_rx_len++)
            buffer[winusb_bulk_rx_len] = winusb_bulk_out[winusb_bulk_rx_len];

        /*
         * Prepare dual-ram buffer for next OUT transaction
         */

        WINUSB_BD_BULK_OUT.Cnt = sizeof(winusb_bulk_out);
        mUSBBufferReady(WINUSB_BD_BULK_OUT);
    }//end if
    
    return winusb_bulk_rx_len;
    
}

/******************************************************************************
 * Function:        void WinUsbBulkTx(char *buffer, byte len)
 *
 * PreCondition:    mWinUsbBulkTxIsBusy() must return false.
 *
 *                  Value of 'len' must be equal to or smaller than
 *                  WINUSB_BULK_IN_EP_SIZE
 *                  For a full-speed bulk endpoint, the largest buffer size is
 *                  64 bytes.
 *
 * Input:           buffer  : Pointer to the starting location of data bytes
 *                  len     : Number of bytes to be transferred
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Use this macro to transfer data located in data memory.
 *
 *                  Remember: mWinUsbBulkTxIsBusy() must return false before user
 *                  can call this function.
 *                  Unexpected behavior will occur if this function is called
 *                  when mWinUsbBulkTxIsBusy() == 0
 *
 *                  Typical Usage:
 *                  if(!mWinUsbBulkTxIsBusy())
 *                      WinUsbBulkTx(buffer, 3);
 *
 * Note:            None
 *****************************************************************************/
void WinUsbBulkTx(char *buffer, byte len)
{
	byte i;
	
    /*
     * Value of len should be equal to or smaller than WINUSB_BULK_IN_EP_SIZE.
     * This check forces the value of len to meet the precondition.
     */

	if(len > WINUSB_BULK_IN_EP_SIZE)
	    len = WINUSB_BULK_IN_EP_SIZE;

   /*
    * Copy data from user's buffer to dual-ram buffer
    */

    for (i = 0; i < len; i++)
    	winusb_bulk_in[i] = buffer[i];

    WINUSB_BD_BULK_IN.Cnt = len;
    mUSBBufferReady(WINUSB_BD_BULK_IN);

}

/******************************************************************************
 * Function:        void WinusbControlReadTransferHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:       	Called on receiving a Setup packet for a WinUSB control Read request.
 * 					Calls a function to provide data to send in the Data stage.	
 *
 * Note:            None
 *****************************************************************************/

void WinusbControlReadTransferHandler(void)
{
	// It's a WinUSB request.

    ctrl_trf_session_owner = MUID_WINUSB;
			
	// Provide the data to send.

	WinusbCtrlTrfTxService();

	// The data is in RAM.

    usb_stat.ctrl_trf_mem = _RAM;               

}//end WinusbControlReadTransferHandler

/******************************************************************************
 * Function:        void WinusbControlWriteTransferHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:       	Called on receiving a Setup packet for a control Write request.
 * 					Defines the location of data to send in the Data stage.
 *
 * Note:            None
 *****************************************************************************/

void WinusbControlWriteTransferHandler(void)
{
	// It's a WinUSB request.

    ctrl_trf_session_owner = MUID_WINUSB;

	// The data that arrives in the Data stage will be stored in    
	// winusb_control_out (defined in usbmmap.h).

    pDst.bRam = (byte*)&winusb_control_out;

} //end WinusbControlWriteTransferHandler

/******************************************************************************
 * Function:        void WinUsbInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        WinUsbInitEP initializes endpoints, buffer descriptors,
 *                  internal state-machine, and variables.
 *                  It should be called after the USB host has sent out a
 *                  SET_CONFIGURATION request.
 *                  See USBStdSetCfgHandler() in usb9.c for examples.
 *
 * Note:            None
 *****************************************************************************/
void WinUsbInitEP(void)
{   
    winusb_interrupt_rx_len =0;
    
	// Endpoint 1 IN and Endpoint 1 OUT handle interrupt transfers.
	// Endpoint 2 IN and Endpoint 2 OUT handle bulk transfers.

    WINUSB_UEP1 = EP_OUT_IN|HSHK_EN;       
    WINUSB_BD_INTERRUPT_OUT.Cnt = sizeof(winusb_interrupt_out);    // Set buffer size
    WINUSB_BD_INTERRUPT_OUT.ADR = (byte*)&winusb_interrupt_out;    // Set buffer address
    WINUSB_BD_INTERRUPT_OUT.Stat._byte = _USIE|_DAT0|_DTSEN; // Set status

	WINUSB_UEP2 = EP_OUT_IN|HSHK_EN; 
 	WINUSB_BD_BULK_OUT.Cnt = sizeof(winusb_bulk_out);    // Set buffer size
    WINUSB_BD_BULK_OUT.ADR = (byte*)&winusb_bulk_out;    // Set buffer address
    WINUSB_BD_BULK_OUT.Stat._byte = _USIE|_DAT0|_DTSEN; // Set status

    /*
     * Do not have to init Cnt of IN pipes here.
     * Reason:  Number of bytes to send to the host
     *          varies from one transaction to
     *          another. Cnt should equal the exact
     *          number of bytes to transmit for
     *          a given IN transaction.
     *          This number of bytes will only
     *          be known right before the data is
     *          sent.
     */

    WINUSB_BD_INTERRUPT_IN.ADR = (byte*)&winusb_interrupt_in;      // Set buffer address
    WINUSB_BD_INTERRUPT_IN.Stat._byte = _UCPU|_DAT1;         // Set status

    WINUSB_BD_BULK_IN.ADR = (byte*)&winusb_bulk_in;      // Set buffer address
    WINUSB_BD_BULK_IN.Stat._byte = _UCPU|_DAT1;         // Set status

}//end WinUsbInitEP

/******************************************************************************
 * Function:        byte WinUsbInterruptRx(char *buffer)
 *
 * PreCondition:    Input argument 'buffer' should point to a buffer area that
 *                  is bigger or equal to the size specified by 'len'.
 *
 * Input:           buffer  : Pointer to where received bytes are to be stored
 *
 * Output:          The number of bytes copied to buffer.
 *
 * Side Effects:    Publicly accessible variable winusb_interrupt_rx_len is updated
 *                  with the number of bytes copied to buffer.
 *                  Once WinUsbInterruptRx is called, subsequent retrieval of
 *                  winusb_interrupt_rx_len can be done by calling macro
 *                  mWinUsbGetInterruptRxLength().
 *
 * Overview:        WinUsbInterruptRx copies a string of bytes received through
 *                  the interrupt OUT endpoint to a user's specified location. 
 *                  It is a non-blocking function. It does not wait
 *                  for data if there is no data available. Instead it returns
 *                  '0' to notify the caller that there is no data available.
 *
 * Note:           
 *****************************************************************************/

byte WinUsbInterruptRx(char *buffer)
{

	byte	len;

    winusb_interrupt_rx_len = 0;
	winusb_bulk_rx_len = 0;
    
    if(!mWinUsbInterruptRxIsBusy())
    {
		// Get the number of received bytes.

        len = WINUSB_BD_INTERRUPT_OUT.Cnt;  
        
        /*
         * Copy data from dual-ram buffer to user's buffer
         */
        for(winusb_interrupt_rx_len = 0; winusb_interrupt_rx_len < len; winusb_interrupt_rx_len++)
            buffer[winusb_interrupt_rx_len] = winusb_interrupt_out[winusb_interrupt_rx_len];

        /*
         * Prepare dual-ram buffer for next OUT transaction
         */
        WINUSB_BD_INTERRUPT_OUT.Cnt = sizeof(winusb_interrupt_out);
        mUSBBufferReady(WINUSB_BD_INTERRUPT_OUT);
    }//end if
    
    return winusb_interrupt_rx_len;
    
}//end WinUsbInterruptRx

/******************************************************************************
 * Function:        void WinUsbInterruptTx(char *buffer, byte len)
 *
 * PreCondition:    mWinUsbInterruptTxIsBusy() must return false.
 *
 *                  Value of 'len' must be equal to or smaller than
 *                  WINUSB_INTERRUPT_IN_EP_SIZE
 *                  For a full-speed interrupt endpoint, the largest buffer size is
 *                  64 bytes.
 *
 * Input:           buffer  : Pointer to the starting location of data bytes
 *                  len     : Number of bytes to be transferred
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Use this macro to transfer data located in data memory.
 *
 *                  Remember: mWinUsbInterruptTxIsBusy() must return false before user
 *                  can call this function.
 *                  Unexpected behavior will occur if this function is called
 *                  when mWinUsbInterruptTxIsBusy() == 0
 *
 *                  Typical Usage:
 *                  if(!mWinUsbInterruptTxIsBusy())
 *                      WinUsbInterruptTx(buffer, 3);
 *
 * Note:            None
 *****************************************************************************/
void WinUsbInterruptTx(char *buffer, byte len)
{
	byte i;
	
    /*
     * Value of len should be equal to or smaller than WINUSB_INTERRUPT_IN_EP_SIZE.
     * This check forces the value of len to meet the precondition.
     */

	if(len > WINUSB_INTERRUPT_IN_EP_SIZE)
	    len = WINUSB_INTERRUPT_IN_EP_SIZE;

   /*
    * Copy data from user's buffer to dual-ram buffer
    */

    for (i = 0; i < len; i++)
    	winusb_interrupt_in[i] = buffer[i];

    WINUSB_BD_INTERRUPT_IN.Cnt = len;
    mUSBBufferReady(WINUSB_BD_INTERRUPT_IN);

}//end WinUsbInterruptTx

#endif 


