/*********************************************************************
 *
 *                Microchip USB C18 Firmware - Mouse Demo
 *
 *********************************************************************
 * FileName:        user_winusb.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original HID mouse application.
 * Jan Axelson           2/20/08    Modified and renamed for WinUSB device.	 
 ********************************************************************/
/*

This application demonstrates communicating with the WinUSB driver
included with Windows XP and Windows Vista.

The firmware performs control, bulk, and interrupt transfers. 

In a control Write transfer, firmware stores received data in a buffer.
In a control Read transfer, firmware sends the data received in the previous control Write transfer.

In a bulk OUT transfer, firmware stores received data in a buffer.
In a bulk IN transfer, firmware sends the data received in the previous bulk OUT transfer.

In an interrupt OUT transfer, firmware stores received data in a buffer.
In an interrupt IN transfer, firmware sends the data received in the previous bulk OUT transfer.

This software and a host application to communicate with the device are available
from www.Lvr.com

Limits:
All of a transfer's data must fit in the endpoint's buffer. No support for multi-transaction bulk 
and interrupt transfers or control transfers with multi-transaction Data stages.
*/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"             
#include "user\user_winusb.h"

/** V A R I A B L E S ********************************************************/
#pragma udata

char bulk_receive_buffer[WINUSB_BULK_OUT_EP_SIZE];
char bulk_transmit_buffer[WINUSB_BULK_IN_EP_SIZE];
char control_receive_buffer[EP0_BUFF_SIZE];
char control_transmit_buffer[EP0_BUFF_SIZE];
char interrupt_receive_buffer[WINUSB_INTERRUPT_OUT_EP_SIZE];
char interrupt_transmit_buffer[WINUSB_INTERRUPT_IN_EP_SIZE];
byte number_of_bytes_read;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
void BulkTransferLoopback(void);
void InterruptTransferLoopback(void);
void WinusbCtrlTrfRxService(void);
void WinusbCtrlTrfTxService(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                mLED_2 = !mLED_1;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

/******************************************************************************
 * Function:        void BulkTransferLoopback(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        	Checks to see if data has been received on 
						the bulk OUT endpoint. If yes, sends the data 
						back via the bulk IN endpoint.
 *
 * Note:            None
 *****************************************************************************/

void BulkTransferLoopback(void)
{
	byte count;

	// Find out if data has been received on the bulk OUT endpoint.

	number_of_bytes_read = WinUsbBulkRx(bulk_receive_buffer);

	if (number_of_bytes_read > 0)
	{
		// Data was received. Send it back.

		for (count = 1; count <= WINUSB_BULK_OUT_BYTES; count = count + 1)
		{				
			bulk_transmit_buffer[count-1] = bulk_receive_buffer[count-1];			
		}
		// Wait until the bulk IN endpoint isn't busy.

		while(mWinUsbBulkTxIsBusy())
		{
			// Service USB interrupts.

			USBDriverService(); 
		}
		// The data will be sent in the next bulk IN transfer.

   	    WinUsbBulkTx(bulk_transmit_buffer, number_of_bytes_read);
	}
}//end BulkTransferLoopback

/******************************************************************************
 * Function:        void InterruptTransferLoopback(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        	Checks to see if data has been received on 
						the interrupt OUT endpoint. If yes, sends the data 
						back via the interrupt IN endpoint.
 *
 * Note:            None
 *****************************************************************************/

void InterruptTransferLoopback(void)
{
	byte count;

	// Find out if data has been received on the interrupt OUT endpoint.

	number_of_bytes_read = WinUsbInterruptRx(interrupt_receive_buffer);

	if (number_of_bytes_read > 0)
	{
		// Data was received. Send it back.

		for (count = 1; count <= WINUSB_INTERRUPT_OUT_BYTES; count = count + 1)
		{				
			interrupt_transmit_buffer[count-1] = interrupt_receive_buffer[count-1];			
		}
		// Wait until the interrupt IN endpoint isn't busy.

		while(mWinUsbInterruptTxIsBusy())
		{
			// Service USB interrupts.

			USBDriverService(); 
		}
		// Data will be sent in the next interrupt IN transfer.

   	    WinUsbInterruptTx(interrupt_transmit_buffer, number_of_bytes_read);
	}
}//end InterruptTransferLoopback

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/

void ProcessIO(void)
{   
	// Called repeatedly in main().

    BlinkUSBStatus();

    // User Application USB tasks

    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

    InterruptTransferLoopback();
    BulkTransferLoopback();

}//end ProcessIO

/******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Initialize user functions
 *
 * Note:            None
 *****************************************************************************/

void UserInit(void)
{
    mInitAllLEDs();

}//end UserInit

/******************************************************************************
 * Function:        void WinusbCtrlTrfRxService(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        If a defined WinUSB control Write request is in progress,  
 *                  get received data in the Data stage.  
 *
 * Note:            None
 *****************************************************************************/

void WinusbCtrlTrfRxService()
{
	byte count;

	// Is it a defined request?

	switch (SetupPkt.bRequest)
    {
		case 0x01: // WinUSB vendor-defined control request 1 
		    {
				// Copy the received Data-stage data to the control Read data buffer 
				// to send back to the host in a control Read transfer.
 				
				for (count = 1; count <= WINUSB_CONTROL_OUT_BYTES; count = count + 1)
				{
					winusb_control_in[count-1] = winusb_control_out[count-1];
				}										
				break;		
			}

		// Add other defined requests here.

		default:
			break;		
	 } 

} // end WinusbCtrlTrfRxService

/******************************************************************************
 * Function:        void WinusbCtrlTrfTxService(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:       	Prepares to send the data in the Data 
 * 					stage of a control Read transfer.
 *
 * Note:            None
 *****************************************************************************/

void WinusbCtrlTrfTxService(void)
{
	byte count;

	// Set pSrc.bRam to point to the report.

	pSrc.bRam = (byte*)&winusb_control_in;

	// The number of bytes in the report (from usbcfg.h).

    LSB(wCount) = WINUSB_CONTROL_IN_BYTES;

} // end WinusbCtrlTrfTxService




